/*
 * $Id: gnet_stats.c,v 1.11 2002/11/07 18:03:31 wyldfire Exp $
 *
 * Copyright (c) 2001-2002, Richard Eckart
 *
 *----------------------------------------------------------------------
 * This file is part of gtk-gnutella.
 *
 *  gtk-gnutella is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  gtk-gnutella is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with gtk-gnutella; if not, write to the Free Software
 *  Foundation, Inc.:
 *      59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *----------------------------------------------------------------------
 */

#include "gnet_stats.h"

static guint8 stats_lut[256];

static guint32 stats_pkg_recv[MSG_TYPE_COUNT];
static guint32 stats_pkg_genr[MSG_TYPE_COUNT];	/* Sent from local node */
static guint32 stats_pkg_rely[MSG_TYPE_COUNT];	/* Relayed by node */
static guint32 stats_pkg_expd[MSG_TYPE_COUNT];
static guint32 stats_pkg_drop[MSG_TYPE_COUNT];

static guint32 stats_byte_recv[MSG_TYPE_COUNT];
static guint32 stats_byte_genr[MSG_TYPE_COUNT];
static guint32 stats_byte_rely[MSG_TYPE_COUNT];
static guint32 stats_byte_expd[MSG_TYPE_COUNT];
static guint32 stats_byte_drop[MSG_TYPE_COUNT];

static guint32 stats_drop_reason[MSG_DROP_REASON_COUNT][MSG_TYPE_COUNT];

static guint32 stats_general[GNR_TYPE_COUNT];

/***
 *** Public functions
 ***/

void gnet_stats_init(void)
{
    memset(stats_lut, MSG_UNKNOWN, sizeof(stats_lut));
    stats_lut[GTA_MSG_INIT] = MSG_INIT;
    stats_lut[GTA_MSG_INIT_RESPONSE]= MSG_INIT_RESPONSE;
    stats_lut[GTA_MSG_BYE] = MSG_BYE;
    stats_lut[GTA_MSG_QRP] = MSG_QRP;
    stats_lut[GTA_MSG_VENDOR] = MSG_VENDOR;
    stats_lut[GTA_MSG_STANDARD] = MSG_STANDARD;
    stats_lut[GTA_MSG_PUSH_REQUEST] = MSG_PUSH_REQUEST;
    stats_lut[GTA_MSG_SEARCH] = MSG_SEARCH;
    stats_lut[GTA_MSG_SEARCH_RESULTS] = MSG_SEARCH_RESULTS;

    memset(stats_pkg_recv, 0, sizeof(stats_pkg_recv));
    memset(stats_pkg_genr, 0, sizeof(stats_pkg_genr));
    memset(stats_pkg_rely, 0, sizeof(stats_pkg_rely));
    memset(stats_pkg_expd, 0, sizeof(stats_pkg_expd));
    memset(stats_pkg_drop, 0, sizeof(stats_pkg_drop));

    memset(stats_byte_recv, 0, sizeof(stats_byte_recv));
    memset(stats_byte_genr, 0, sizeof(stats_byte_genr));
    memset(stats_byte_rely, 0, sizeof(stats_byte_rely));
    memset(stats_byte_expd, 0, sizeof(stats_byte_expd));
    memset(stats_byte_drop, 0, sizeof(stats_byte_drop));

    memset(stats_general, 0, sizeof(stats_general));

    memset(stats_drop_reason, 0, 
        sizeof(guint32)*MSG_DROP_REASON_COUNT*MSG_TYPE_COUNT);
}

/*
 * gnet_stats_count_received_header
 *
 * Called when Gnutella header has been read.
 */
void gnet_stats_count_received_header(gnutella_node_t *n)
{
    n->received++;

    stats_pkg_recv[MSG_TOTAL]++;
    stats_pkg_recv[stats_lut[n->header.function]]++;
    stats_byte_recv[MSG_TOTAL] += sizeof(n->header);
    stats_byte_recv[stats_lut[n->header.function]] += sizeof(n->header);
}

/*
 * gnet_stats_count_received_payload
 *
 * Called when Gnutella payload has been read.
 */
void gnet_stats_count_received_payload(gnutella_node_t *n)
{
    guint32 size = n->size;

    stats_byte_recv[MSG_TOTAL] += size;
    stats_byte_recv[stats_lut[n->header.function]] += size;
}

void gnet_stats_count_sent(
	gnutella_node_t *n, guint8 type, guint8 hops, guint32 size)
{
	guint32 *stats_pkg = hops ? stats_pkg_rely : stats_pkg_genr;
	guint32 *stats_byte = hops ? stats_byte_rely : stats_byte_genr;

    stats_pkg[MSG_TOTAL]++;
    stats_pkg[stats_lut[type]]++;
    stats_byte[MSG_TOTAL] += size;
    stats_byte[stats_lut[type]] += size;
}

void gnet_stats_count_expired(gnutella_node_t *n)
{
    guint32 size = n->size + sizeof(n->header);

    stats_pkg_expd[MSG_TOTAL]++;
    stats_pkg_expd[stats_lut[n->header.function]]++;
    stats_byte_expd[MSG_TOTAL] += size;
    stats_byte_expd[stats_lut[n->header.function]] += size;
}

#define DROP_STATS(s) do {											\
    if (															\
        (reason == MSG_DROP_ROUTE_LOST) ||							\
        (reason == MSG_DROP_DUPLICATE) ||							\
        (reason == MSG_DROP_NO_ROUTE)								\
    )																\
        stats_general[GNR_ROUTING_ERRORS]++;						\
																	\
    stats_drop_reason[reason][MSG_TOTAL]++;							\
    stats_drop_reason[reason][stats_lut[n->header.function]]++;		\
    stats_pkg_drop[MSG_TOTAL]++;									\
    stats_pkg_drop[stats_lut[n->header.function]]++;				\
    stats_byte_drop[MSG_TOTAL] += (s);								\
    stats_byte_drop[stats_lut[n->header.function]] += (s);			\
} while (0)

void gnet_stats_count_dropped(gnutella_node_t *n, msg_drop_reason_t reason)
{
    guint32 size = n->size + sizeof(n->header);

	DROP_STATS(size);
}

void gnet_stats_count_general(gnutella_node_t *n, gint type, guint32 amount)
{
    stats_general[type] += amount;
}

void gnet_stats_count_dropped_nosize(
	gnutella_node_t *n, msg_drop_reason_t reason)
{
	DROP_STATS(sizeof(n->header));
}

/***
 *** Public functions (gnet.h)
 ***/

void gnet_stats_get(gnet_stats_t *s)
{
    g_assert(s != NULL);

    memcpy(s->pkg.received, stats_pkg_recv, sizeof(s->pkg.received));
    memcpy(s->pkg.generated, stats_pkg_genr, sizeof(s->pkg.generated));
    memcpy(s->pkg.relayed, stats_pkg_rely, sizeof(s->pkg.relayed));
    memcpy(s->pkg.expired, stats_pkg_expd, sizeof(s->pkg.expired));
    memcpy(s->pkg.dropped, stats_pkg_drop, sizeof(s->pkg.dropped));

    memcpy(s->byte.received, stats_byte_recv, sizeof(s->byte.received));
    memcpy(s->byte.generated, stats_byte_genr, sizeof(s->byte.generated));
    memcpy(s->byte.relayed, stats_byte_rely, sizeof(s->byte.relayed));
    memcpy(s->byte.expired, stats_byte_expd, sizeof(s->byte.expired));
    memcpy(s->byte.dropped, stats_byte_drop, sizeof(s->byte.dropped));

    memcpy(s->drop_reason, stats_drop_reason, sizeof(s->drop_reason));

    memcpy(s->general, stats_general, sizeof(s->general));
}

